@@ -6,6 +6,10 @@
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
+#if HAVE_STRINGS_H
+#include <strings.h>
+#endif /* HAVE_STRINGS_H */
+
 #if HAVE_STDLIB_H 
 #include <stdlib.h>
 #endif  /* HAVE_STDLIB_H */
@@ -16,6 +20,7 @@
 #include "tgdb.h"
 #include "interface.h"
 #include "tokenizer.h"
+#include "highlight_groups.h"
 #include "cgdb.h"
 
 extern struct tgdb *tgdb;
@@ -42,7 +47,7 @@ enum ConfigType
     CONFIG_TYPE_FUNC_VOID,
     CONFIG_TYPE_FUNC_BOOL,
     CONFIG_TYPE_FUNC_INT,
-    CONFIG_TYPE_FUNC_STRING,
+    CONFIG_TYPE_FUNC_STRING
 };
 
 static int command_set_arrowstyle( const char *value );
@@ -59,7 +64,7 @@ static struct ConfigVariable
     enum ConfigType type;
     void *data;
 } VARIABLES[] = {
-    // keep this stuff sorted! !sort
+    /* keep this stuff sorted! !sort*/
     /* arrowstyle */
     { "arrowstyle", "as", CONFIG_TYPE_FUNC_STRING, command_set_arrowstyle},
 
@@ -102,36 +107,43 @@ static int command_focus_cgdb( void );
 static int command_focus_gdb( void );
 static int command_focus_tty( void );
 
+static int command_do_bang( void );
 static int command_do_continue( void );
 static int command_do_finish( void );
 static int command_do_help( void );
 static int command_do_next( void );
 static int command_do_quit( void );
 static int command_do_run( void );
+static int command_do_shell( void );
 static int command_do_step( void );
 static int command_source_reload( void );
 
 static int command_parse_syntax( void );
+static int command_parse_highlight (void);
 
 typedef int (*action_t)(void);
 static struct commands
 {
     const char *name;
-    // these functions will return 0 on success and 1 on error.  
-    // Should the configuration file processing continue after an error?
+    /* these functions will return 0 on success and 1 on error.  */
+    /* Should the configuration file processing continue after an error?*/
     action_t action;
 } COMMANDS[] = {
-    // keep this stuff sorted, you can use !sort in vi
+    /* keep this stuff sorted, you can use !sort in vi*/
+    /* bang        */ { "bang",        command_do_bang },
     /* continue    */ { "continue",    command_do_continue },
     /* edit        */ { "e",           command_source_reload },
     /* edit        */ { "edit",        command_source_reload },
     /* finish      */ { "finish",      command_do_finish },
     /* help        */ { "help",        command_do_help },
+    /* highlight   */ { "hi",          command_parse_highlight },
+    /* highlight   */ { "highlight",   command_parse_highlight },
     /* insert      */ { "insert",      command_focus_gdb },
     /* next        */ { "next",        command_do_next },
     /* q           */ { "q",           command_do_quit },
     /* quit        */ { "quit",        command_do_quit },
     /* run         */ { "run",         command_do_run },
+    /* shell       */ { "shell",       command_do_shell },
     /* step        */ { "step",        command_do_step },
     /* syntax      */ { "syntax",      command_parse_syntax },
 };
@@ -222,7 +234,7 @@ int command_set_winsplit( const char *value )
       if_set_winsplit( WIN_SPLIT_BOTTOM_FULL );
    } else {
       if_set_winsplit( WIN_SPLIT_EVEN );
-   } // end if
+   } /* end if*/
 
    return 0;
 }
@@ -269,6 +281,11 @@ int command_focus_tty( void )
     return 0;
 }
 
+int command_do_bang( void )
+{
+    return 0;
+}
+
 int command_do_continue( void )
 {
     tgdb_request_ptr request_ptr;
@@ -328,6 +345,11 @@ int command_do_run( void )
     return 0;
 }
 
+int command_do_shell( void )
+{
+    return run_shell_command(NULL);
+}
+
 int command_do_step( void )
 {
     tgdb_request_ptr request_ptr;
@@ -387,6 +409,12 @@ int command_parse_syntax( void )
   return 0;
 }
 
+int 
+command_parse_highlight (void)
+{
+  return hl_groups_parse_config (hl_groups_instance);
+}
+
 int command_parse_set( void )
 {
     /* commands could look like the following:
@@ -399,7 +427,7 @@ int command_parse_set( void )
     int rv = 1;
     int boolean = 1;
     const char * value = NULL;
-    
+
     switch ( (rv = yylex()) ) {
     case EOL: {
         /* TODO: Print out all the variables that have been set. */
@@ -583,7 +611,7 @@ int command_parse_file( FILE *fp )
     while( linenumber++, fgets( p, sizeof(buffer) - ( p - buffer ), fp ) )
     {
         int bufferlen = strlen( buffer );
-        if( buffer[bufferlen - 2] == '\\' ) 
+        if ((bufferlen-2>=0) && buffer[bufferlen - 2] == '\\' ) 
         {
             /* line continuation character, read another line into the buffer */
             linenumber--;
