@@ -273,13 +273,13 @@ int a2_err_msg ( void *ctx ) {
 int a2_is_client_ready(void *ctx) {
 	struct annotate_two *a2 = (struct annotate_two *)ctx;
     if ( !a2->tgdb_initialized )
-        return FALSE;
+        return 0;
 
     /* If the user is at the prompt */
     if ( data_get_state(a2->data) == USER_AT_PROMPT )
-        return TRUE;
+        return 1;
 
-    return FALSE;
+    return 0;
 }
 
 int a2_parse_io ( 
@@ -336,17 +336,21 @@ a2_get_source_filename_pair (void *ctx, const char *file )
 }
 
 int
-a2_get_current_location (void *ctx)
+a2_get_current_location (void *ctx, int on_startup)
 {
   struct annotate_two *a2 = (struct annotate_two *)ctx;
   int ret;
 
-  ret = commands_issue_command (a2->c, a2->client_command_list, ANNOTATE_LIST,
-	                        NULL, 0);
-  if (ret == -1)
+  if (on_startup)
   {
-    logger_write_pos ( logger, __FILE__, __LINE__, "commands_issue_command error");
-    return -1;
+    ret = commands_issue_command (a2->c, a2->client_command_list, ANNOTATE_LIST,
+				  NULL, 0);
+    if (ret == -1)
+    {
+      logger_write_pos (logger, __FILE__, __LINE__, 
+			"commands_issue_command error");
+      return -1;
+    }
   }
 
   ret = commands_issue_command (a2->c, a2->client_command_list,
@@ -427,7 +431,7 @@ int a2_user_ran_command ( void *ctx ) {
 	return commands_user_ran_command ( a2->c, a2->client_command_list );
 }
 
-int a2_prepare_for_command ( void *ctx, struct tgdb_client_command *com ) {
+int a2_prepare_for_command ( void *ctx, struct tgdb_command *com ) {
 	struct annotate_two *a2 = (struct annotate_two *)ctx;
 	return commands_prepare_for_command ( a2, a2->c, com );
 }
