@@ -25,7 +25,6 @@
 
 #include "tgdb.h"
 #include "tgdb_command.h"
-#include "tgdb_client_command.h"
 #include "tgdb_client_interface.h"
 #include "fs_util.h"
 #include "ibuf.h"
@@ -67,20 +66,51 @@ struct tgdb
 
   /***************************************************************************
    * All the queue's the clients can run commands through
+   * The different queue's can be slightly confusing.
    **************************************************************************/
 
-  /** The commands that the client has requested to run */
-  struct queue *gdb_client_request_queue;
-
   /**
-   The commands that need to be run through gdb. 
-   Examples are 'b main', 'run', ...  */
+   * The commands that need to be run through GDB.
+   *
+   * This is a buffered queue that represents all of the commands that TGDB
+   * needs to execute. These commands will be executed one at a time. That is,
+   * 1 command will be issued, and TGDB will wait for the entire response before
+   * issuing any more commands. It is even possible that while this command is
+   * executing, that the client context will add commands to the 
+   * oob_command_queue. If this happens TGDB will execute all of the commands
+   * in the oob_command_queue before executing the next command in this queue. 
+   */
   struct queue *gdb_input_queue;
 
+  /** 
+   * The commands that the client has requested to run.
+   *
+   * TGDB buffers all of the commands that the FE wants to run. That is,
+   * if the FE is sending commands faster than TGDB can pass the command to
+   * GDB and get a response, then the commands are buffered until it is there
+   * turn to run. These commands are run in the order that they are recieved.
+   *
+   * This is here as a convience to the FE. TGDB currently does not access 
+   * these commands. It provides the push/pop functionality and it erases the
+   * queue when a control_c is received.
+   */
+  struct queue *gdb_client_request_queue;
+
   /** 
    * The out of band input queue. 
-   * These commands should *always* be run first 
-   * This is a list of commands that TGDB needs to run in order to satisfy itself.
+   *
+   * These commands are used by the client context. When a single client context 
+   * command is run, sometimes it will discover that it needs to run other commands
+   * in order to satisfy the functionality requested by the GUI. For instance, a 
+   * single GUI function request could take N client context commands to generate
+   * a response. Also, to make things worse, sometimes the client context doesn't 
+   * know if it will take 1 or N commands to satisfy the request until after it has 
+   * sent and recieved the information from the first command. Thus, while the 
+   * client context is processing the first command, it may add more commands to 
+   * this queue to specify that these commands need to be run before any other 
+   * commands are sent to GDB.
+   *
+   * These commands should *always* be run first.
    */
   struct queue *oob_input_queue;
 
@@ -136,10 +166,10 @@ struct tgdb
 /* }}} */
 
 /* Temporary prototypes {{{ */
-static int tgdb_deliver_command (struct tgdb *tgdb, int fd,
+static int tgdb_deliver_command (struct tgdb *tgdb,
 				 struct tgdb_command *command);
-static int tgdb_run_command (struct tgdb *tgdb);
-static int tgdb_dispatch_command (struct tgdb *tgdb,
+static int tgdb_unqueue_and_deliver_command (struct tgdb *tgdb);
+static int tgdb_run_or_queue_command (struct tgdb *tgdb,
 				  struct tgdb_command *com);
 
 /* }}} */
@@ -158,10 +188,8 @@ tgdb_process_client_commands (struct tgdb *tgdb)
 {
   struct tgdb_list *client_command_list;
   tgdb_list_iterator *iterator;
-  struct tgdb_client_command *client_command;
   struct tgdb_command *command;
   char *command_data = NULL;
-  int add_command;
 
   client_command_list = tgdb_client_get_client_commands (tgdb->tcc);
   iterator = tgdb_list_get_first (client_command_list);
@@ -169,45 +197,17 @@ tgdb_process_client_commands (struct tgdb *tgdb)
   while (iterator)
     {
       enum tgdb_command_choice command_choice;
-      enum tgdb_command_action_choice action_choice =
-	TGDB_COMMAND_ACTION_NONE;
 
-      client_command =
-	(struct tgdb_client_command *) tgdb_list_get_item (iterator);
-      add_command = 0;
+      command =
+	(struct tgdb_command *) tgdb_list_get_item (iterator);
 
-      switch (client_command->command_choice)
+      if (tgdb_run_or_queue_command (tgdb, command) == -1)
 	{
-	case TGDB_CLIENT_COMMAND_NORMAL:
-	  command_choice = TGDB_COMMAND_TGDB_CLIENT;
-	  add_command = 1;
-	  break;
-	case TGDB_CLIENT_COMMAND_PRIORITY:
-	  command_choice = TGDB_COMMAND_TGDB_CLIENT_PRIORITY;
-	  add_command = 1;
-	  break;
-	default:
 	  logger_write_pos (logger, __FILE__, __LINE__,
-			    "unknown switch case");
+			    "tgdb_run_or_queue_command failed");
 	  return -1;
 	}
 
-      if (add_command)
-	{
-	  command = tgdb_command_create (command_data,
-					 command_choice,
-					 action_choice,
-					 client_command,
-					 0 /* No CONSOLE commands here */ );
-
-	  if (tgdb_dispatch_command (tgdb, command) == -1)
-	    {
-	      logger_write_pos (logger, __FILE__, __LINE__,
-				"tgdb_dispatch_command failed");
-	      return -1;
-	    }
-	}
-
       iterator = tgdb_list_next (iterator);
     }
 
@@ -330,7 +330,7 @@ tgdb_initialize_logger_interface (struct tgdb *tgdb, char *config_dir)
   return 0;
 }
 
-// Createing and Destroying a libtgdb context. {{{
+/* Createing and Destroying a libtgdb context. {{{*/
 
 struct tgdb *
 tgdb_initialize (const char *debugger,
@@ -353,20 +353,12 @@ tgdb_initialize (const char *debugger,
       return NULL;
     }
 
-  /* Initialize the command queue's */
-
-  /* initialize client request buffer */
-  tgdb->gdb_client_request_queue = queue_init ();
-
-  /* initialize users buffer */
-  tgdb->gdb_input_queue = queue_init ();
-
-  /* initialize the out of band queue */
-  tgdb->oob_input_queue = queue_init ();
+  tgdb->gdb_client_request_queue  = queue_init ();
+  tgdb->gdb_input_queue		  = queue_init ();
+  tgdb->oob_input_queue		  = queue_init ();
 
   tgdb->tcc = tgdb_client_create_context (debugger, argc, argv, config_dir,
 					  TGDB_CLIENT_DEBUGGER_GNU_GDB,
-//                      TGDB_CLIENT_PROTOCOL_GNU_GDB_GDBMI,
 					  TGDB_CLIENT_PROTOCOL_GNU_GDB_ANNOTATE_TWO,
 					  logger);
 
@@ -394,19 +386,6 @@ tgdb_initialize (const char *debugger,
   *debugger_fd = tgdb->debugger_stdout;
   *inferior_fd = tgdb->inferior_stdout;
 
-  if (1)
-    {
-
-      /*a2_setup_io( ); */
-    }
-  else
-    {
-      /*init_gdbmi (); */
-      logger_write_pos (logger, __FILE__, __LINE__,
-			"GDB does not support annotations");
-      return NULL;
-    }
-
   return tgdb;
 }
 
@@ -428,7 +407,7 @@ tgdb_shutdown (struct tgdb *tgdb)
   return tgdb_client_destroy_context (tgdb->tcc);
 }
 
-// }}}
+/* }}}*/
 
 char *
 tgdb_err_msg (struct tgdb *tgdb)
@@ -463,7 +442,7 @@ tgdb_client_modify_breakpoint_call (struct tgdb *tgdb, const char *file,
  * is when the debugger is ready and there are no commands to run.
  *
  * \return
- * TRUE if can issue directly to gdb. Otherwise FALSE.
+ * 1 if can issue directly to gdb. Otherwise 0.
  */
 static int
 tgdb_can_issue_command (struct tgdb *tgdb)
@@ -471,42 +450,23 @@ tgdb_can_issue_command (struct tgdb *tgdb)
   if (tgdb->IS_SUBSYSTEM_READY_FOR_NEXT_COMMAND &&
       tgdb_client_is_client_ready (tgdb->tcc) &&
       (queue_size (tgdb->gdb_input_queue) == 0))
-    return TRUE;
+    return 1;
 
-  return FALSE;
+  return 0;
 }
 
 /**
  * Determines if tgdb has commands it needs to run.
  *
  * \return
- * TRUE if can issue directly to gdb. Otherwise FALSE.
+ * 1 if can issue directly to gdb. Otherwise 0.
  */
 static int
 tgdb_has_command_to_run (struct tgdb *tgdb)
 {
-  if (tgdb_client_is_client_ready (tgdb->tcc) && ((queue_size
-						   (tgdb->gdb_input_queue) >
-						   0)
-						  ||
-						  (queue_size
-						   (tgdb->oob_input_queue) >
-						   0)))
-    return TRUE;
-
-  return FALSE;
-}
-
-/**
- * As of now, I don't know what this is used for.
- *
- * \return
- * 1 if ready, or 0 if not ready
- */
-int
-is_ready (struct tgdb *tgdb)
-{
-  if (tgdb_can_issue_command (tgdb))
+  if (tgdb_client_is_client_ready (tgdb->tcc) && 
+	  ((queue_size (tgdb->gdb_input_queue) > 0) ||
+	   (queue_size (tgdb->oob_input_queue) > 0)))
     return 1;
 
   return 0;
@@ -516,20 +476,13 @@ int
 tgdb_is_busy (struct tgdb *tgdb, int *is_busy)
 {
   /* Validate parameters */
-  if (!tgdb)
+  if (!tgdb || !is_busy)
     {
       logger_write_pos (logger, __FILE__, __LINE__, "tgdb_is_busy failed");
       return -1;
     }
 
-  if (!is_busy)
-    {
-
-      logger_write_pos (logger, __FILE__, __LINE__, "tgdb_is_busy failed");
-      return -1;
-    }
-
-  if (tgdb_can_issue_command (tgdb) == TRUE)
+  if (tgdb_can_issue_command (tgdb) == 1)
     *is_busy = 0;
   else
     *is_busy = 1;
@@ -591,7 +544,8 @@ tgdb_handle_signals (struct tgdb *tgdb)
  ******************************************************************************/
 
 /* 
- * Sends a command to the debugger.
+ * Sends a command to the debugger. This function gets called when the GUI
+ * wants to run a command.
  *
  * \param tgdb
  * An instance of the tgdb library to operate on.
@@ -600,19 +554,16 @@ tgdb_handle_signals (struct tgdb *tgdb)
  * This is the command that should be sent to the debugger.
  *
  * \param command_choice
- * This tells tgdb_send who is sending the command. Currently, if readline 
- * gets a command from the user, it calls this function. Also, this function
- * gets called usually when the GUI tries to run a command.
+ * This tells tgdb_send who is sending the command.
  *
  * \return
  * 0 on success, or -1 on error.
  */
 static int
-tgdb_send (struct tgdb *tgdb,
-	   char *command, enum tgdb_command_choice command_choice)
+tgdb_send (struct tgdb *tgdb, char *command, 
+	   enum tgdb_command_choice command_choice)
 {
 
-  struct tgdb_client_command *tcc;
   struct tgdb_command *tc;
   struct ibuf *temp_command = ibuf_init ();
   int length = strlen (command);
@@ -624,22 +575,15 @@ tgdb_send (struct tgdb *tgdb,
     ibuf_addchar (temp_command, '\n');
 
   /* Create the client command */
-  tcc = tgdb_client_command_create (ibuf_get (temp_command),
-				    TGDB_CLIENT_COMMAND_NORMAL,
-				    TGDB_CLIENT_COMMAND_DISPLAY_COMMAND_AND_RESULTS,
-				    TGDB_CLIENT_COMMAND_ACTION_NONE, NULL);
+  tc = tgdb_command_create (ibuf_get (temp_command), command_choice, NULL);
 
   ibuf_free (temp_command);
   temp_command = NULL;
 
-  /* Create the TGDB command */
-  tc = tgdb_command_create (NULL,
-			    command_choice, TGDB_COMMAND_ACTION_NONE, tcc, 0);
-
-  if (tgdb_dispatch_command (tgdb, tc) == -1)
+  if (tgdb_run_or_queue_command (tgdb, tc) == -1)
     {
       logger_write_pos (logger, __FILE__, __LINE__,
-			"tgdb_dispatch_command failed");
+			"tgdb_run_or_queue_command failed");
       return -1;
     }
 
@@ -656,78 +600,71 @@ tgdb_send (struct tgdb *tgdb,
 }
 
 /**
- * This will dispatch a command to be run now if ready, otherwise,
- * the command will be put in a queue to run later.
+ * If TGDB is ready to process another command, then this command will be
+ * sent to the debugger. However, if TGDB is not ready to process another command,
+ * then the command will be queued and run when TGDB is ready.
  *
  * \param tgdb
  * The TGDB context to use.
  *
  * \param command
- * The command to dispatch
+ * The command to run or queue.
  *
  * \return
  * 0 on success or -1 on error
  */
 static int
-tgdb_dispatch_command (struct tgdb *tgdb, struct tgdb_command *command)
+tgdb_run_or_queue_command (struct tgdb *tgdb, struct tgdb_command *command)
 {
-
   int ret = 0;
+  int can_issue;
 
-  switch (command->command_choice)
-    {
-    case TGDB_COMMAND_FRONT_END:
-    case TGDB_COMMAND_TGDB_CLIENT:
-      if (tgdb_can_issue_command (tgdb))
-	ret = tgdb_deliver_command (tgdb, tgdb->debugger_stdin, command);
-      else
-	queue_append (tgdb->gdb_input_queue, command);
-      break;
+  can_issue = tgdb_can_issue_command (tgdb);
 
-    case TGDB_COMMAND_CONSOLE:
-      if (tgdb_can_issue_command (tgdb))
-	ret = tgdb_deliver_command (tgdb, tgdb->debugger_stdin, command);
-      else
-	return -3;
-      break;
-
-    case TGDB_COMMAND_TGDB_CLIENT_PRIORITY:
-      if (tgdb_can_issue_command (tgdb))
-	ret = tgdb_deliver_command (tgdb, tgdb->debugger_stdin, command);
-      else
-	queue_append (tgdb->oob_input_queue, command);
-      break;
-    default:
-      logger_write_pos (logger, __FILE__, __LINE__, "unimplemented command");
+  if (can_issue)
+  {
+    if (tgdb_deliver_command (tgdb, command) == -1)
       return -1;
-    }
-
-  if (ret == -2)
-    return -2;
+  } else {
+    /* Make sure to put the command into the correct queue. */
+    switch (command->command_choice)
+      {
+      case TGDB_COMMAND_FRONT_END:
+      case TGDB_COMMAND_TGDB_CLIENT:
+	queue_append (tgdb->gdb_input_queue, command);
+	break;
+      case TGDB_COMMAND_TGDB_CLIENT_PRIORITY:
+	queue_append (tgdb->oob_input_queue, command);
+	break;
+      case TGDB_COMMAND_CONSOLE:
+	logger_write_pos (logger, __FILE__, __LINE__, "unimplemented command");
+	return -1;
+	break;
+      default:
+	logger_write_pos (logger, __FILE__, __LINE__, "unimplemented command");
+	return -1;
+      }
+  }
 
   return 0;
 }
 
 /** 
- *  Sends a command to the debugger  
+ * Will send a command to the debugger immediatly. No queueing will be done
+ * at this point.
  *
  * \param tgdb
  * The TGDB context to use.
  *
- * \param fd 
- * the file descriptor to write to ( debugger's input )
- *
  * \param command 
- * the command to run
+ * The command to run.
  *
  *  NOTE: This function assummes valid commands are being sent to it. 
  *        Error checking should be done before inserting into queue.
  */
 static int
-tgdb_deliver_command (struct tgdb *tgdb, int fd, struct tgdb_command *command)
+tgdb_deliver_command (struct tgdb *tgdb, struct tgdb_command *command)
 {
-  struct tgdb_client_command *client_command = command->client_command;
-
   tgdb->IS_SUBSYSTEM_READY_FOR_NEXT_COMMAND = 0;
 
   /* Here is where the command is actually given to the debugger.
@@ -737,21 +674,17 @@ tgdb_deliver_command (struct tgdb *tgdb, int fd, struct tgdb_command *command)
    * debugger is being given.
    */
   if (command->command_choice == TGDB_COMMAND_FRONT_END)
-    tgdb->last_gui_command =
-      xstrdup (client_command->tgdb_client_command_data);
+    tgdb->last_gui_command = xstrdup (command->tgdb_command_data);
 
   /* A command for the debugger */
-  if (tgdb_client_prepare_for_command (tgdb->tcc, client_command) == -1)
-    {
-      return -2;
-    }
+  if (tgdb_client_prepare_for_command (tgdb->tcc, command) == -1)
+    return -1;
 
   /* A regular command from the client */
-  io_debug_write_fmt ("<%s>", client_command->tgdb_client_command_data);
+  io_debug_write_fmt ("<%s>", command->tgdb_command_data);
 
-  io_writen (fd,
-	     client_command->tgdb_client_command_data,
-	     strlen (client_command->tgdb_client_command_data));
+  io_writen (tgdb->debugger_stdin, command->tgdb_command_data,
+	     strlen (command->tgdb_command_data));
 
   /* Uncomment this if you wish to see all of the commands, that are 
    * passed to GDB. */
@@ -771,16 +704,16 @@ tgdb_deliver_command (struct tgdb *tgdb, int fd, struct tgdb_command *command)
 }
 
 /**
- * Sends to gdb the next command.
+ * TGDB will search it's command queue's and determine what the next command
+ * to deliever to GDB should be.
  *
  * \return
- * 0 on normal termination ( command was run )
- * 2 if the queue was cleared because of ^c
+ * 0 on success, -1 on error
  */
 static int
-tgdb_run_command (struct tgdb *tgdb)
+tgdb_unqueue_and_deliver_command (struct tgdb *tgdb)
 {
-tgdb_run_command_tag:
+tgdb_unqueue_and_deliver_command_tag:
 
   /* This will redisplay the prompt when a command is run
    * through the gui with data on the console.
@@ -794,10 +727,10 @@ tgdb_run_command_tag:
        */
       struct tgdb_command *item = NULL;
       item = queue_pop (tgdb->oob_input_queue);
-      tgdb_deliver_command (tgdb, tgdb->debugger_stdin, item);
+      tgdb_deliver_command (tgdb, item);
       tgdb_command_destroy (item);
-      /* If the queue is not empty, run a command */
     }
+  /* If the queue is not empty, run a command */
   else if (queue_size (tgdb->gdb_input_queue) > 0)
     {
       struct tgdb_command *item = NULL;
@@ -806,19 +739,19 @@ tgdb_run_command_tag:
       /* If at the misc prompt, don't run the internal tgdb commands,
        * In fact throw them out for now, since they are only 
        * 'info breakpoints' */
-      if (tgdb_client_can_tgdb_run_commands (tgdb->tcc) == TRUE)
+      if (tgdb_client_can_tgdb_run_commands (tgdb->tcc) == 1)
 	{
 	  if (item->command_choice != TGDB_COMMAND_CONSOLE)
 	    {
 	      tgdb_command_destroy (item);
-	      goto tgdb_run_command_tag;
+	      goto tgdb_unqueue_and_deliver_command_tag;
 	    }
 	}
 
       /* This happens when a command was skipped because the client no longer
        * needs the command to be run */
-      if (tgdb_deliver_command (tgdb, tgdb->debugger_stdin, item) == -2)
-	goto tgdb_run_command_tag;
+      if (tgdb_deliver_command (tgdb, item) == -1)
+	goto tgdb_unqueue_and_deliver_command_tag;
 
       tgdb_command_destroy (item);
     }
@@ -826,21 +759,6 @@ tgdb_run_command_tag:
   return 0;
 }
 
-/**
- * This is called when readline says it has recieved a full line.
- */
-static int
-tgdb_command_callback (struct tgdb *tgdb, const char *line)
-{
-  struct ibuf *command = ibuf_init ();
-  ibuf_add (command, line);
-  ibuf_addchar (command, '\n');
-  tgdb_send (tgdb, ibuf_get (command), TGDB_COMMAND_CONSOLE);
-  ibuf_free (command);
-  command = NULL;
-  return 0;
-}
-
 /* These functions are used to communicate with the inferior */
 int
 tgdb_send_inferior_char (struct tgdb *tgdb, char c)
@@ -854,7 +772,7 @@ tgdb_send_inferior_char (struct tgdb *tgdb, char c)
   return 0;
 }
 
-/* tgdb_tty_recv: returns to the caller data from the child */
+/* returns to the caller data from the child */
 size_t
 tgdb_recv_inferior_data (struct tgdb * tgdb, char *buf, size_t n)
 {
@@ -976,6 +894,7 @@ tgdb_process (struct tgdb *tgdb, char *buf, size_t n, int *is_finished)
   char local_buf[10 * n];
   ssize_t size;
   size_t buf_size = 0;
+  int is_busy;
 
   /* make the queue empty */
   tgdb_delete_responses (tgdb);
@@ -992,6 +911,15 @@ tgdb_process (struct tgdb *tgdb, char *buf, size_t n, int *is_finished)
   if (tgdb->last_gui_command != NULL)
     {
       int ret;
+
+      if (tgdb_is_busy (tgdb, &is_busy) == -1)
+	{
+	  logger_write_pos (logger, __FILE__, __LINE__, 
+			    "tgdb_is_busy failed");
+	  return -1;
+	}
+      *is_finished = !is_busy;
+
       if (tgdb->show_gui_commands)
 	{
 	  strcpy (buf, tgdb->last_gui_command);
@@ -1096,7 +1024,7 @@ tgdb_process (struct tgdb *tgdb, char *buf, size_t n, int *is_finished)
 
   /* 4. runs the users buffered command if any exists */
   if (tgdb_has_command_to_run (tgdb))
-    tgdb_run_command (tgdb);
+    tgdb_unqueue_and_deliver_command (tgdb);
 
 tgdb_finish:
 
@@ -1105,28 +1033,17 @@ tgdb_finish:
    */
   tgdb->command_list_iterator = tgdb_list_get_first (tgdb->command_list);
 
-  if (is_finished)
+  if (tgdb_is_busy (tgdb, &is_busy) == -1)
     {
-      int is_busy;
-      int val;
-      val = tgdb_is_busy (tgdb, &is_busy);
-      if (val == -1)
-        {
-          logger_write_pos (logger, __FILE__, __LINE__,
-			    "tgdb_handle_signals failed");
-	  return -1;
-	}
-
-      if (is_busy)
-	*is_finished = 0;
-      else
-	*is_finished = 1;
+      logger_write_pos (logger, __FILE__, __LINE__, "tgdb_is_busy failed");
+      return -1;
     }
+  *is_finished = !is_busy;
 
   return buf_size;
 }
 
-// Getting Data out of TGDB {{{
+/* Getting Data out of TGDB {{{*/
 
 struct tgdb_response *
 tgdb_get_response (struct tgdb *tgdb)
@@ -1156,9 +1073,9 @@ tgdb_delete_responses (struct tgdb *tgdb)
   tgdb_list_free (tgdb->command_list, tgdb_types_free_command);
 }
 
-// }}}
+/* }}}*/
 
-// Inferior tty commands {{{
+/* Inferior tty commands {{{*/
 
 int
 tgdb_tty_new (struct tgdb *tgdb)
@@ -1178,11 +1095,11 @@ tgdb_tty_name (struct tgdb *tgdb)
   return tgdb_client_get_tty_name (tgdb->tcc);
 }
 
-// }}}
+/* }}}*/
 
 /* Functional commands {{{ */
 
-// Request {{{
+/* Request {{{*/
 
 tgdb_request_ptr
 tgdb_request_run_console_command (struct tgdb *tgdb, const char *command)
@@ -1249,7 +1166,7 @@ tgdb_request_filename_pair (struct tgdb *tgdb, const char *file)
 }
 
 tgdb_request_ptr 
-tgdb_request_current_location (struct tgdb *tgdb)
+tgdb_request_current_location (struct tgdb *tgdb, int on_startup)
 {
   tgdb_request_ptr request_ptr;
   if (!tgdb)
@@ -1261,6 +1178,7 @@ tgdb_request_current_location (struct tgdb *tgdb)
     return NULL;
 
   request_ptr->header = TGDB_REQUEST_CURRENT_LOCATION;
+  request_ptr->choice.current_location.on_startup = on_startup;
 
   return request_ptr;
 }
@@ -1324,35 +1242,42 @@ tgdb_request_complete (struct tgdb *tgdb, const char *line)
   return request_ptr;
 }
 
-// }}}
+/* }}}*/
 
-// Process {{{
+/* Process {{{*/
 
-int
+static int
 tgdb_process_console_command (struct tgdb *tgdb, tgdb_request_ptr request)
 {
   int ret;
+  struct ibuf *command;
 
   if (!tgdb || !request)
     return -1;
 
-  if (!is_ready (tgdb))
+  if (!tgdb_can_issue_command (tgdb))
     return -1;
 
   if (request->header != TGDB_REQUEST_CONSOLE_COMMAND)
     return -1;
   
-  ret = tgdb_command_callback (tgdb, request->choice.console_command.command);
-  if (ret == -1)
-    {
-      logger_write_pos (logger, __FILE__, __LINE__, "tgdb_send_debugger_data");
+  command = ibuf_init ();
+  ibuf_add (command, request->choice.console_command.command);
+  ibuf_addchar (command, '\n');
+
+  if (tgdb_send (tgdb, ibuf_get (command), TGDB_COMMAND_CONSOLE) == -1)
+  {
+      logger_write_pos (logger, __FILE__, __LINE__, "tgdb_send failed");
       return -1;
-    }
+  }
+
+  ibuf_free (command);
+  command = NULL;
 
   return 0;
 }
 
-int
+static int
 tgdb_process_info_sources (struct tgdb *tgdb, tgdb_request_ptr request)
 {
   int ret;
@@ -1369,7 +1294,7 @@ tgdb_process_info_sources (struct tgdb *tgdb, tgdb_request_ptr request)
   return ret;
 }
 
-int
+static int
 tgdb_process_filename_pair (struct tgdb *tgdb, tgdb_request_ptr request)
 {
   int ret;
@@ -1386,7 +1311,7 @@ tgdb_process_filename_pair (struct tgdb *tgdb, tgdb_request_ptr request)
   return ret;
 }
 
-int 
+static int 
 tgdb_process_current_location (struct tgdb *tgdb, tgdb_request_ptr request)
 {
   int ret = 0;
@@ -1397,13 +1322,14 @@ tgdb_process_current_location (struct tgdb *tgdb, tgdb_request_ptr request)
   if (request->header != TGDB_REQUEST_CURRENT_LOCATION)
     return -1;
 
-  ret = tgdb_client_get_current_location (tgdb->tcc);
+  ret = tgdb_client_get_current_location (
+	  tgdb->tcc, request->choice.current_location.on_startup);
   tgdb_process_client_commands (tgdb);
   
   return ret;
 }
 
-int
+static int
 tgdb_process_debugger_command (struct tgdb *tgdb, tgdb_request_ptr request)
 {
   if (!tgdb || !request)
@@ -1416,7 +1342,7 @@ tgdb_process_debugger_command (struct tgdb *tgdb, tgdb_request_ptr request)
 		    TGDB_COMMAND_FRONT_END);
 }
 
-int
+static int
 tgdb_process_modify_breakpoint (struct tgdb *tgdb, tgdb_request_ptr request)
 {
   char *val;
@@ -1447,7 +1373,7 @@ tgdb_process_modify_breakpoint (struct tgdb *tgdb, tgdb_request_ptr request)
   return 0;
 }
 
-int
+static int
 tgdb_process_complete (struct tgdb *tgdb, tgdb_request_ptr request)
 {
   int ret;
@@ -1470,7 +1396,7 @@ tgdb_process_command (struct tgdb *tgdb, tgdb_request_ptr request)
   if (!tgdb || !request)
     return -1;
 
-  if (!is_ready (tgdb))
+  if (!tgdb_can_issue_command (tgdb))
     return -1;
 
   if (request->header == TGDB_REQUEST_CONSOLE_COMMAND)
@@ -1491,11 +1417,11 @@ tgdb_process_command (struct tgdb *tgdb, tgdb_request_ptr request)
   return 0;
 }
 
-// }}}
+/* }}}*/
 
 /* }}} */
 
-// TGDB Queue commands {{{
+/* TGDB Queue commands {{{*/
 
 int 
 tgdb_queue_append (struct tgdb *tgdb, tgdb_request_ptr request)
@@ -1532,32 +1458,28 @@ tgdb_queue_size (struct tgdb *tgdb, int *size)
   return 0;
 }
 
-// }}}
+/* }}}*/
 
-// Signal Handling Support {{{
+/* Signal Handling Support {{{*/
 
 int
 tgdb_signal_notification (struct tgdb *tgdb, int signum)
 {
-  pid_t pid;
+  struct termios t;
+  cc_t *sig_char = NULL;
 
-  pid = tgdb_client_get_debugger_pid (tgdb->tcc);
-
-  if (pid == -1)
-    return -1;
+  tcgetattr (tgdb->debugger_stdin, &t);
 
   if (signum == SIGINT)
     {				/* ^c */
       tgdb->control_c = 1;
-      kill (pid, SIGINT);
-    }
-  else if (signum == SIGTERM)
-    {
-      kill (pid, SIGTERM);
+      sig_char = &t.c_cc[VINTR];
+      write (tgdb->debugger_stdin, sig_char, 1);
     }
   else if (signum == SIGQUIT)
     {				/* ^\ */
-      kill (pid, SIGQUIT);
+      sig_char = &t.c_cc[VQUIT];
+      write (tgdb->debugger_stdin, sig_char, 1);
     }
   else if (signum == SIGCHLD)
     {
@@ -1567,9 +1489,9 @@ tgdb_signal_notification (struct tgdb *tgdb, int signum)
   return 0;
 }
 
-// }}}
+/* }}}*/
 
-// Config Options {{{
+/* Config Options {{{*/
 int
 tgdb_set_verbose_gui_command_output (struct tgdb *tgdb, int value)
 {
@@ -1597,4 +1519,4 @@ tgdb_set_verbose_error_handling (struct tgdb *tgdb, int value)
   return logger_is_recording (logger);
 }
 
-// }}}
+/* }}}*/
